#!/bin/bash
###############################################################################
# eVotUM - Electronic Voting System
#
# gen_eVotUM_certrequest.sh
#
# Bash script that generates an eVotUM certificate request (CSR).
#
#
# Copyright (c) 2016 Universidade do Minho
# Developed by José Pina Miranda - Devise Futures, Lda. (jose.miranda@devisefutures.com)
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################

# Begin Configuration - CHANGE AT YOUR OWN RISK -
PKIPATH='/etc/evotum/pki'
ROOT_SERIAL_FILE="${PKIPATH}/certserial/ECroot.serial"
# End Configuration

### Verify Root CA
# Verifies if $ROOT_SERIAL_FILE exists and get SERIAL number
if [ -f $ROOT_SERIAL_FILE ]; then
  ROOT_SERIAL=$(<$ROOT_SERIAL_FILE)
else
  echo "CA root serial file ($ROOT_SERIAL_FILE) non existent. Certificate request cannot be generated."
  exit
fi

ROOT_CERT_FILE="${PKIPATH}/certs/ca.cert.${ROOT_SERIAL}.pem"

# Verifies if $ROOT_CERT_FILE exists
if [ ! -f $ROOT_CERT_FILE ]; then
  echo "CA certificate file ($ROOT_CERT_FILE) doesn't exist. Certificate request cannot be generated."
  exit
fi


### Select certificate request to generate

echo "Select one of the eVotUM certificate request (CSR) to generate:"
echo "  1 - Counter Service"
echo "  2 - Validator Service"
echo "  3 - Voting System"
echo "  4 - Election Management System"
echo "  5 - Web Server"
echo "  6 - System Administrator"
echo -n "?: "
read kp

case $kp in
        1 )
            KP_TEXT="Counter Service"
            PREFIX='counter_service'
            ;;
        2 )
            KP_TEXT="Validator Service"
            PREFIX='validator_service'
            ;;
        3 )
            KP_TEXT="Voting System"
            PREFIX='voting_system'
            ;;
        4 )
            KP_TEXT="Election Management System"
            PREFIX='election_mgt_system'
            ;;
        5 )
            KP_TEXT="Web Server"
            PREFIX='web_server'
            echo "> Please insert common name (typically composed of Host + Domain Name) - max 64 characters"
            echo -n "?: "
            read COMMON_NAME
            export COMMON_NAME
            ;;
        6 )
            KP_TEXT="System Administrator"
            PREFIX='system_administrator'
            echo "> Please insert System Administrator name - max 64 characters"
            echo -n "?: "
            read COMMON_NAME
            export COMMON_NAME
            ;;
        *)
            echo "Invalid input"
            exit 1
            ;;
esac

### Generate certificate request for selected type
SERIAL_FILE="${PKIPATH}/certserial/${PREFIX}.serial"

# Verifies if $SERIAL_FILE exists and get SERIAL number
if [ -f $SERIAL_FILE ]; then
  export SERIAL=$(<$SERIAL_FILE)
else
  echo "Serial file $SERIAL_FILE doesn't exist. Certificate request cannot be generated."
  exit
fi

CONF_FILE="${PKIPATH}/config/req_${PREFIX}.cnf"
KEY_FILE="${PKIPATH}/private/${PREFIX}.key.${SERIAL}.pem"
CSR_FILE="${PKIPATH}/csr/${PREFIX}.csr.${SERIAL}.pem"


# Generate certificate request
printf "Generate $KP_TEXT certificate request to $CSR_FILE\n"
gen_CSR.sh $KEY_FILE $CONF_FILE $CSR_FILE
