#!/bin/bash
###############################################################################
# eVotUM - Electronic Voting System
#
# gen_eVotUM_keypair.sh
#
# Bash script that generates an eVotUM key pair (except for the root CA).
#
#
# Copyright (c) 2016 Universidade do Minho
# Developed by José Pina Miranda - Devise Futures, Lda. (jose.miranda@devisefutures.com)
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################

# Begin Configuration - CHANGE AT YOUR OWN RISK -
KEY_BITS=2048
EC_CURVE=prime256v1
PKIPATH='/etc/evotum/pki'
ROOT_SERIAL_FILE="${PKIPATH}/certserial/ECroot.serial"
# End Configuration

### Verify Root CA
# Verifies if $ROOT_SERIAL_FILE exists and get SERIAL number
if [ -f $ROOT_SERIAL_FILE ]; then
  ROOT_SERIAL=$(<$ROOT_SERIAL_FILE)
else
  echo "CA root serial file ($ROOT_SERIAL_FILE) non existent. Key pair cannot be generated."
  exit
fi

ROOT_CERT_FILE="${PKIPATH}/certs/ca.cert.${ROOT_SERIAL}.pem"

# Verifies if $ROOT_CERT_FILE exists
if [ ! -f $ROOT_CERT_FILE ]; then
  echo "CA certificate file ($ROOT_CERT_FILE) doesn't exist. Key pair cannot be generated."
  exit
fi


### Select key pair to generate

echo "Select one of the eVotUM key pairs to generate:"
echo "  1 - Counter Service"
echo "  2 - Validator Service"
echo "  3 - Voting System"
echo "  4 - Election Management System"
echo "  5 - Web Server"
echo "  6 - System Administrator"
echo -n "?: "
read kp

case $kp in
        1 )
            KP_TEXT="Counter Service"
            PREFIX='counter_service'
            ;;
        2 )
            KP_TEXT="Validator Service"
            PREFIX='validator_service'
            ;;
        3 )
            KP_TEXT="Voting System"
            PREFIX='voting_system'
            ;;
        4 )
            KP_TEXT="Election Management System"
            PREFIX='election_mgt_system'
            ;;
        5 )
            KP_TEXT="Web Server"
            PREFIX='web_server'
            ;;
        6 )
            KP_TEXT="System Administrator"
            PREFIX='system_administrator'
            ;;
        *)
            echo "Invalid input"
            exit 1
            ;;
esac

### Generate key pair for selected type
SERIAL_FILE="${PKIPATH}/certserial/${PREFIX}.serial"

# Verifies if $SERIAL_FILE exists and get SERIAL number
if [ -f $SERIAL_FILE ]; then
  SERIAL=$(($(<$SERIAL_FILE) + 1))
else
  SERIAL=1
fi

echo $SERIAL > $SERIAL_FILE
KEY_FILE="${PKIPATH}/private/${PREFIX}.key.${SERIAL}.pem"

if [[ $kp == 2 ]]; then # Validator service must generate elliptic curve key pairs
  printf "Generate $KP_TEXT key pair to $KEY_FILE\n"
  gen_ECkeypair.sh $EC_CURVE $KEY_FILE
  exit
fi

# Generate key pair
printf "Generate $KP_TEXT key pair to $KEY_FILE\n"
gen_RSAkeypair.sh $KEY_BITS $KEY_FILE
