#!/bin/bash
###############################################################################
# eVotUM - Electronic Voting System
#
# gen_rootCA_certificate.sh
#
# Bash script that issues root CA self-signed certificate.
#
#
# Copyright (c) 2016 Universidade do Minho
# Developed by José Pina Miranda - Devise Futures, Lda. (jose.miranda@devisefutures.com)
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################

# Begin Configuration - CHANGE AT YOUR OWN RISK -
DAYS=3652   # 10 years = 3652 days
PKIPATH='/etc/evotum/pki'
SERIAL_FILE="${PKIPATH}/certserial/ECroot.serial"
CONF="${PKIPATH}/config/req_root.cnf"
# End Configuration

# Verifies if $SERIAL_FILE exists and get SERIAL number
if [ -f $SERIAL_FILE ]; then
  export SERIAL=$(<$SERIAL_FILE)
else
  echo $SERIAL_FILE non existent. Certificate cannot be issued.
  exit
fi

KEY_FILE="${PKIPATH}/private/ca.key.${SERIAL}.pem"

# Verifies if $KEY_FILE exists
if [ ! -f $KEY_FILE ]; then
  echo Key pair $KEY_FILE non existent. Certificate cannot be issued.
  exit
fi

CERT_FILE="${PKIPATH}/certs/ca.cert.${SERIAL}.pem"

# Verifies if $CERT_FILE exists
if [ -f $CERT_FILE ]; then
  echo Certificate file $CERT_FILE already existent. Certificate cannot be issued.
  exit
fi

# Issue N $days self-signed CA certificate
printf "Issue self-signed CA root certificate - validity $DAYS days (10 years = 3652 days) - to $CERT_FILE\n"
openssl req -config $CONF -key $KEY_FILE -new -x509 -days $DAYS -extensions v3_ca -out $CERT_FILE
chmod 444 $CERT_FILE

# Visualize issued certificate
printf "#########################################################\n"
printf "Certificate details ($CERT_FILE)\n"
printf "#########################################################\n"
openssl x509 -noout -text -in $CERT_FILE
