#!/bin/bash
###############################################################################
# eVotUM - Electronic Voting System
#
# gen_CSR.sh
#
# Bash script that creates the certificate request (CSR) to $3 - key pair in $1 and configuration file in $2 -.
#
#
# Copyright (c) 2016 Universidade do Minho
# Developed by José Pina Miranda - Devise Futures, Lda. (jose.miranda@devisefutures.com)
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################

# Begin Configuration - CHANGE AT YOUR OWN RISK -

# End Configuration
###############################################################################
# start bash

if [  $# -lt 3 ]; then
    echo Usage: $0 "<key_pair_file> <configuration_file> <output_file>"
    printf "Creates the certificate request (CSR) to output_file, from key pair (in key_pair_file) and with configuration in configuration_file.\n"
    exit
fi

KEY=$1
CONF=$2
CSR=$3

# Verifies if output_file exists
if [ -f $CSR ]; then
    echo Output file $CSR already existent. Certificate request cannot be generated.
    exit
fi

# Verifies if key pair file exists
if [ ! -f $KEY ]; then
    echo "Key pair file $KEY doesn't exist. Certificate request cannot be generated."
    exit
fi

# Verifies if configuration file exists
if [ ! -f $CONF ]; then
    echo "Configuration file $CONF doesn't exist. Certificate request cannot be generated."
    exit
fi

# Create CSR
openssl req -config $CONF -key $KEY -new -out $CSR

# View details of CSR
printf "##########################################################\n"
printf "Certificate request (CSR) details ($CSR)\n"
printf "##########################################################\n"
openssl req -noout -text -in $CSR
