#!/bin/bash
###############################################################################
# eVotUM - Electronic Voting System
#
# gen_ECkeypair.sh
#
# Bash script that generates an elliptic curve (EC) key pair, using $1 curve and writes it to $2 in PEM format.
#
#
# Copyright (c) 2016 Universidade do Minho
# Developed by José Pina Miranda - Devise Futures, Lda. (jose.miranda@devisefutures.com)
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################

# Begin Configuration - CHANGE AT YOUR OWN RISK -
PARAM_FILE=/tmp/$$.prm
# End Configuration
###############################################################################
# start bash

if [  $# -lt 2 ]; then
    echo Usage: $0 "<curve_name> <output_file>"
    echo Generates an elliptic curve key pair using curve curve_name and writes it to output_file in PEM format
    exit
fi

CURVE=$1
KEY=$2

# Verifies if output_file exists
if [ -f $KEY ]; then
    echo Output file $KEY already existent. Key pair cannot be generated.
    exit
fi

# Create elliptic curve parameters:
openssl ecparam -out $PARAM_FILE -name $CURVE

# Generate key pair
openssl genpkey -paramfile $PARAM_FILE -aes-128-cbc -out $KEY
chmod 400 $KEY

# Remove parameter file
rm -f $PARAM_FILE
