#!/bin/bash
###############################################################################
# eVotUM - Electronic Voting System
#
# gen_certificate.sh
#
# Bash script that issues the certificate to $3 - certificate request in $1 and root CA certificate in $2 -.
#
#
# Copyright (c) 2016 Universidade do Minho
# Developed by José Pina Miranda - Devise Futures, Lda. (jose.miranda@devisefutures.com)
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################

# Begin Configuration - CHANGE AT YOUR OWN RISK -
PKIPATH='/etc/evotum/pki'
CA_SIGN="${PKIPATH}/config/ca_root.cnf"
# End Configuration
###############################################################################
# start bash

if [  $# -lt 3 ]; then
    echo Usage: $0 "<certificate_request_file> <root_CA_certificate_file> <output_file>"
    printf "Issues the certificate to output_file, from certificate request (in certificate_request_file) and with root CA certificate in root_CA_certificate_file.\n"
    exit
fi

CERT=$3
CSR=$1
CA_CERT=$2

# Verifies if output_file exists
if [ -f $CERT ]; then
    echo "Output file $CERT already existent. Certificate cannot be issued."
    exit
fi

# Verifies if certificate request file exists
if [ ! -f $CSR ]; then
    echo "Certificate request file $CSR doesn't exist. Certificate cannot be issued."
    exit
fi

# Verifies if root CA certificate file exists
if [ ! -f $CA_CERT ]; then
    echo "Root CA certificate file $CA_CERT doesn't exist. Certificate cannot be issued."
    exit
fi

# Issue certificate
openssl ca -config $CA_SIGN -extensions server_cert -notext -in $CSR -out $CERT -batch
chmod 444 $CERT

# Verify chain of trust
printf "\n#############################################################################################\n"
printf "Verify chain of trust ($CERT):\n"
openssl verify -CAfile $CA_CERT $CERT

# View details of issued certificate
printf "\n#############################################################################################\n"
printf "(Optional) To view the details of the issued certificate, please enter the following command:\n"
printf "openssl x509 -noout -text -in $CERT\n"
